#ifndef WPIPE_WRESAMPLER_HH
#define WPIPE_WRESAMPLER_HH

// WRESAMPLE Resample time series data at a common sample frequency
//
// WRESAMPLE resamples the given time series data at the specified
// common sample frequency.  The input time series data must have the
// same length in time and the ratio of input sample frequencies must
// be expressable as a ratio of integers.
//
// usage:
//
//   resampledData = wresample(data, sampleFrequencies, sampleFrequency);
//
//   data                 cell array of input time series data
//   sampleFrequencies    vector of sample frequencies of input data [Hz]
//   sampleFrequency      desired sample frequency of resampled data [Hz]
//
//   resampledData        cell array of resampled time series data
//
// Note that any decrease in the sample frequency of a channel requires
// the application of a low pass filter to prevent aliasing at the new
// Nyquist frequency.  This filtering is performed using a zero phase
// filter to avoid introducing unwanted time delays into the resampled
// data stream.  However, filter transients are still expected at the
// beginning and end of the resample time series.  In addition, the
// resampled data stream has limited high frequency content since the
// anti-alias filter has a cutoff frequency of approximately 85 percent
// of the new Nyquist frequency.  It is up to the user to account for
// these effects.
//
// WRESAMPLE uses the Matlab RESAMPLE function in the Signal Processing
// Toolbox.
//
// As a first stab, implement for only one input series and support only 
// the reduction of powers of two. Use DecimateBy2
//
// See also RESAMPLE, WCONDITION, and WSEARCH.

// Shourov K. Chatterji <shourov@ligo.mit.edu>

// $Id: wresample.m 2753 2010-02-26 21:33:24Z jrollins $
#include "wtypes.hh"
#include <map>

class Pipe;

namespace wpipe {

  class resampler {
  public:
    resampler(void);
    ~resampler(void);
    tser_vect wresample(const tser_vect& data, double sampleFrequency);
    Pipe* filter(long from, long to);
  private:
    struct rate_pair {
      rate_pair(long from, long to);
      bool operator<(const rate_pair& x) const;
      long _from;
      long _to;
    };
    typedef std::map<rate_pair, Pipe*> filt_map;
    typedef filt_map::value_type filt_node;
    typedef filt_map::iterator  filt_iter;
    filt_map rate_map;
  };

  //====================================  Inline methods.
  inline 
  resampler::rate_pair::rate_pair(long from, long to) 
    : _from(from), _to(to) 
  {}

  inline bool 
  resampler::rate_pair::operator<(const rate_pair& x) const {
    return _from < x._from || (_from == x._from && _to < x._to);
  }


} // namespace wpipe
#endif // !defined(WPIPE_WRESAMPLER_HH)
